//************************************************************************
// T6963.C
// LowLevel Routinen fr LCD-Displays mit T6963
// based on Instruction of Steve Lawther (Writing Software for T6963C based Graphic LCDs)
//************************************************************************

#include "T6963.h"
#include <avr/pgmspace.h>
unsigned char LCD_CMD;          	// Command-Byte
unsigned char LCD_STATUS;       	// Status-Byte
unsigned char LCD_DATA;         	// Daten-Byte

unsigned char key_data=0;			// eingelesenes Byte vom Tastenport 
/**** LCD-Display-Routinen **********************************************/
void lcd_init(void)
        {
        unsigned int t;
		LCD_CTRL = 0xff;
		DDRC=0xff;
		PORTA=0xFF;
        LCD_RST_L();          // Display Reset
                for (t=0; t<1000; t++)
                {
				asm volatile("nop\n\t"						//Kurze Pause
				 ::);					
				 }              // ca. 125s. Reset
        LCD_RST_H();          // Display Reset fertig
        LCD_DATA = 0x00;        // Text Home Adress setzen
        lcd_send_data();        // Auf 0000H setzen
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x40;
        lcd_send_cmd();
        LCD_DATA = 0x15;        // Text AREA Adress setzen
        lcd_send_data();        // 15H = 16 Zeichen
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x41;
        lcd_send_cmd();
        LCD_DATA = 0x00;        // Grafik Home Adress setzen
        lcd_send_data();        // Auf 0200H setzen
        LCD_DATA = 0x02;
        lcd_send_data();
        LCD_CMD = 0x42;
        lcd_send_cmd();
        LCD_DATA = 0x16;        // Grafik AREA Adress setzen
        lcd_send_data();        // 15H = 16 Zeichen
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x43;
        lcd_send_cmd();
        LCD_DATA = 0x02;        // Offset Pointer setzen
        lcd_send_data();
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x22;
        lcd_send_cmd();
        lcd_send_cmd();
        LCD_CMD = 0x80;         // OR-Mode
        lcd_send_cmd();
        LCD_CMD = 0x9C;         // Text an Grafik an
        lcd_send_cmd();
        LCD_DATA = 0x00;        // Adress Pointer setzen
        lcd_send_data();        // 0000H
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x24;
        lcd_send_cmd();
        lcd_clear_ram();        // zum Schlu noch das Ram des
        }                       // Display's lschen


void lcd_status(void)
        {
        DATA_DIR_IN();          // Port A als Eingang
        LCD_CD_H();             // Commando High
        LCD_RD_L();             // Read Low
        LCD_CE_L();             // Chip Enable Low
        LCD_STATUS = LCD_READ;  // Status lesen
        LCD_CE_H();             // Chip Enable High
		LCD_RD_H();             // Read High
        LCD_STATUS &= (0x03);   // Bits ausmaskieren
        }

void lcd_status_am(void)
        {
        DATA_DIR_IN();          // Port 2 als Eingang
        LCD_CD_H();             // Commando High
        LCD_RD_L();             // Read Low
        LCD_CE_L();             // Chip Enable Low
        LCD_STATUS = LCD_READ;  // Status lesen
        LCD_CE_H();             // Chip Enable High
		LCD_RD_H();             // Read High
        LCD_STATUS &= (0x08);   // Bits ausmaskieren
        }

void lcd_send_cmd(void)         // Commando an Display senden        
        {
            do
                {
                  lcd_status();
                }
            while (LCD_STATUS != 0x03);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = LCD_CMD;
        LCD_CD_H();             // Commando High
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
		LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_send_data(void)         // Daten an Display senden        
        {
            do
                {
                  lcd_status();
                }
            while (LCD_STATUS != 0x03);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = LCD_DATA;
        LCD_CD_L();             // Commando High
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
        LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_send_data_am(void)         // Daten an Display senden
        {
            do
                {
                  lcd_status_am();
                }
            while (LCD_STATUS != 0x08);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = LCD_DATA;
        LCD_CD_L();             // Daten Low
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
        LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_printp(const char *progmem_s, char adress_h, char adress_l)
        {
        char c;

        LCD_DATA = adress_l;    // bergebene Adresse setzen
        lcd_send_data();        
        LCD_DATA = adress_h;
        lcd_send_data();
        LCD_CMD = 0x24;
        lcd_send_cmd();
        LCD_CMD = 0xB0;         // Auto Write Mode an (Auto Inkrement)
        lcd_send_cmd();
	
		while ( (c = pgm_read_byte(progmem_s++)))
			{
			lcd_send_str(c);
			}
        LCD_CMD = 0xB2;         // Auto Write Mode aus (Auto Inkrement)
        lcd_send_cmd();
        }

		
void lcd_print(char *txt, char adress_h, char adress_l)
        {
        char c;

        LCD_DATA = adress_l;    // bergebene Adresse setzen
        lcd_send_data();        
        LCD_DATA = adress_h;
        lcd_send_data();
        LCD_CMD = 0x24;
        lcd_send_cmd();

        while((c = *txt)) //Solange Zeichen grer 0x00
			{
			txt++; //Nchstes Zeichen
			if (c == 0) break; //Raus wenn Zeichen gleich 0
			lcd_send_str(c); 
			LCD_CMD = 0xc0;
			lcd_send_cmd();
			}
        }
		
void lcd_send_str(unsigned char lcd_dat)
        {
        lcd_dat -= 0x20;        // 20h subtrahieren da Display nich ASCII
        LCD_DATA = lcd_dat;     // kompatibel
        lcd_send_data_am();     // Daten an Display senden
        }

void lcd_clear_ram(void)
        {
        unsigned int t;
        LCD_DATA = 0x00;        // int. Ram des Display's lschen.
        lcd_send_data();        // laut Datenblatt des Herstellers 8kb
        LCD_DATA = 0x00;
        lcd_send_data();
        LCD_CMD = 0x24;
        lcd_send_cmd();
        LCD_CMD = 0xB0;         // Auto Write Mode an (Auto Inkrement)
        lcd_send_cmd();
                for (t=0; t<8192; t++)
                {
                LCD_DATA = 0x00;
                lcd_send_data_am();     // Daten an Display senden
                }
        LCD_CMD = 0xB2;         // Auto Write Mode aus (Auto Inkrement)
        lcd_send_cmd();
        }
void lcd_goto_xy(unsigned char adress_h, unsigned char adress_l)
	{
	LCD_DATA = adress_l;    // bergebene Adresse setzen
    lcd_send_data();        
    LCD_DATA = adress_h;
    lcd_send_data();
    LCD_CMD = 0x24;
	}
	
void lcd_printg(const char *progmem_s, char adress_h, char adress_l) 
	{ 
	unsigned int length =1408; //< dieses ist die Bildgre in Bytes 
	unsigned char c; 
	LCD_DATA = adress_l; // bergebene Adresse setzen 
	lcd_send_data(); 
	LCD_DATA = adress_h; 
	lcd_send_data(); 
	LCD_CMD = 0x24; 
	lcd_send_cmd(); 
	LCD_CMD = 0xB0; // Auto Write Mode an (Auto Inkrement) 
	lcd_send_cmd(); 

	while (length !=0)         //es wird ein Bild in der Gre 128*64 Pixel
        {                   //mit Font 6*8 angezeigt
		c = pgm_read_byte(progmem_s++);
		lcd_send_strg(c);
		length--;
		}
	LCD_CMD = 0xB2; // Auto Write Mode aus (Auto Inkrement) 
	lcd_send_cmd(); 
	} 

void lcd_send_strg(unsigned char lcd_dat) 
	{ 
	LCD_DATA = lcd_dat; // kompatibel 
	lcd_send_data(); // Daten an Display senden 
	} 

void set_pixel(int column, int row)
	{
	int addr;
	addr = 0x200 + (row*22)+(column/6);
	LCD_DATA = (addr%256);
	lcd_send_data();
	LCD_DATA = (addr>>8);
	lcd_send_data();
	LCD_CMD = 0x24;
	lcd_send_cmd();
	LCD_CMD = (0xf8 | (5-(column%6)) );
	lcd_send_cmd();
	}

/*****************************************************************************
Name:	IntToAsciiHex   
Parameters:	
		dest_string
			Pointer to a buffer will the string will reside
		min_digits
			Specifies the minimum number of characters the output string will
			have. Leading zeros will be written as '0' characters.
Returns:
		A pointer to the string's NULL character in the string that was just
		created.
Description: 
		This function is used to convert a passed unsigned int into a ASCII
		string represented in Hexidecimal format.
*****************************************************************************/
char * IntToAsciiHex(char * dest_string,int min_digits,unsigned int value)
{
	unsigned int i, total_digits = 0;
	char buff[4];
	
	for(i=0;i<4;i++)
	{
		buff[i] = (char)(value & 0x0F);
		value = value >> 4;
		if( buff[i] <= 9)
			buff[i] += '0';
		else
			buff[i] = (char)(buff[i] - 0xA + 'A');

		if(buff[i] != '0')
			total_digits = i+1;
	}

	if( total_digits < 	min_digits)
		total_digits = min_digits;

	i = total_digits;
	while(i)
	{
		*dest_string++ = buff[i-1];
		i--;
	}

	*dest_string = 0;

	return dest_string;
}
/*****************************************************************************
Name:	IntToAsciiDec   
Parameters:	
		dest_string
			Pointer to a buffer will the string will reside
		min_digits
			Specifies the minimum number of characters the output string will
			have. Leading zeros will be written as '0' characters.
Returns:
		A pointer to the string's NULL character in the string that was just
		created.
Description: 
		This function is used to convert a passed unsigned int into a ASCII
		string represented in base 10 decimal format.
*****************************************************************************/
char * IntToAsciiDec(char * dest_string,int min_digits,unsigned int value)
{
	const unsigned long base10[] = {1,10,100,1000,10000,100000};

	unsigned int tmp;
	unsigned int i, total_digits = 0;
	char buff[5];
	
	for(i=0;i<5;i++)
	{
		tmp = (int)( value % base10[i+1] );
		value -= tmp;

		buff[i] = (char)( tmp / base10[i] );
		buff[i] += '0';
		if(buff[i] != '0')
			total_digits = i+1;
	}
	if( total_digits < 	min_digits)
		total_digits = min_digits;
	i = total_digits;
	while(i)
	{
		*dest_string++ = buff[i-1];
		i--;
	}
	*dest_string = 0;
	return dest_string;
}

